<?php
/** Загрузить маску из файла
 *
 * @param string $file
 * @return array
 */
function RecogLoadMaskFromFile($file){
	$tmp=file($file);
	$mask=array();

	foreach($tmp as $line){
		$line=trim($line);
		if(empty($line))continue;

		$mask[]=str_split($line, 1);
	}

	return $mask;
}

/** Нормализовать вырезанные участки с символами, т.е.:
 * - обрезать лишниее
 * - растянуть на заданный размер
 *
 * @param array $sumbols массив фрагментов масок с символами
 * @return array
 */
function RecogSumbolsNormilize($sumbols){
	$new_sumbols=array();
	for($i=0, $x=count($sumbols); $i<$x; $i++){
		$sumb_mask=$sumbols[$i];

		if(!RecogSpotGetXY($sumb_mask, $x0, $y0, $x1, $y1)){
			imagepngout(imagemerge(image_string('Ошибка границ'), RecogMaskToImage($sumb_mask)));
		}
		$sumb_mask=RecogCutMask($sumb_mask, $x0, $y0, $x1, $y1);

		$sumb=RecogMaskToImage($sumb_mask);

		$newimage=imagecreate(17, 27);
		$color_white=imagecolorallocate($newimage, 255, 255, 255);
		imagefilledrectangle($newimage, 0, 0, imagesx($newimage), imagesy($newimage), $color_white);

		$k = min(17/imagesx($sumb), 27/imagesy($sumb));
		$new_width = imagesx($sumb) * $k;
		$new_height = imagesy($sumb) * $k;

		imagecopyresampled($newimage, $sumb, 0, 0, 0, 0, $new_width, $new_height, imagesx($sumb), imagesy($sumb));
		$new_sumbols[]=RecogImageToMask($newimage);
	}

	return $new_sumbols;
}

/** Разделить полученную маску изображения на несколько символов
 *
 * @param array $mask маска изображения
 * @param integer $count колличество символов
 * @return array массив масок по колличеству символов (array of masks)
 */
function RecogExplodeSumbols($mask, $count){
	$sumbols=array();

	for($sb_i=0; $sb_i<$count; $sb_i++){
		$sumbols[$sb_i]=array();
	}

	$slen=15;
	$rlen=15+(count($mask[0])-($count*15));
	if($rlen<$slen)$rlen=15;

	for($sb_i=0; $sb_i<$count; $sb_i++){
		for($iy=0, $y=count($mask); $iy<$y; $iy++){
			$sumbols[$sb_i][$iy]=array();

			for($ix=0; $ix<$rlen; $ix++){
				if(isset($mask[$iy][$ix+($sb_i*$slen)])){
					$sumbols[$sb_i][$iy][]=$mask[$iy][$ix+($sb_i*$slen)];
				}else{
					$sumbols[$sb_i][$iy][]=0;
				}
			}

		}
	}

	return $sumbols;
}

/** Определить колличество символов на капче по ширине фрагмента с символами
 *
 * @param array $mask
 * @return int
 */
function RecogGetCountSumbols($mask){
	if(count($mask[0])>65){
		return 5;
	}else{
		return 4;
	}
}

/** Преобразовать изображение в маску
 *
 * @param resource $image
 * @return array
 */
function RecogImageToMask($image){
	$mask=array();

	for($iy=0, $y=imagesy($image); $iy<$y; $iy++){
		$mask[$iy]=array();

		for($ix=0, $x=imagesx($image); $ix<$x; $ix++){
			$index=imagecolorat($image, $ix, $iy);
			$color=imagecolorsforindex($image, $index);

			if($color['green']>125 && $color['red']>125 && $color['blue']>=125){
				$mask[$iy][$ix]=0;
			}else{
				$mask[$iy][$ix]=1;
			}
		}
	}

	return $mask;
}

/** Преобразовать маску в изображение
 *
 * @param array $mask
 * @return resource
 */
function RecogMaskToImage($mask){
	$image=imagecreate(count($mask[0]), count($mask));

	$color_white=imagecolorallocate($image, 255, 255, 255);
	$color_black=imagecolorallocate($image, 0, 0, 0);

	for($iy=0, $y=imagesy($image); $iy<$y; $iy++){

		for($ix=0, $x=imagesx($image); $ix<$x; $ix++){
			if($mask[$iy][$ix]==1){
				imagesetpixel($image, $ix, $iy, $color_black);
			}
		}
	}

	return $image;
}

/** Вырезать фрагмент из маски изображения
 *
 * @param array $mask маска изображения
 * @param int $x0 координата
 * @param int $y0 координата
 * @param int $x1 координата
 * @param int $y1 координата
 * @return array новая маска вырезанного фрагмента
 */
function RecogCutMask($mask, $x0, $y0, $x1, $y1){
	$nmask=array();
	$niy=0;

	for($iy=$y0; $iy<$y1; $iy++){
		$nix=0;
		for($ix=$x0; $ix<$x1; $ix++){
			$nmask[$niy][$nix++]=$mask[$iy][$ix];
		}
		$niy++;
	}

	return $nmask;
}

/** Преобразовать изображение в монохромную маску
 *
 * @param resource $image
 * @return array
 */
function RecogGenMonohromeMaskImage($image){
	//Определяем самые часто используемые цвета
	$percision=10;
	$colors=array();
	for($iy=0, $y=imagesy($image); $iy<$y; $iy++){
		for($ix=0, $x=imagesx($image); $ix<$x; $ix++){
			$index=imagecolorat($image, $ix, $iy);
			$color=imagecolorsforindex($image, $index);

			if( ($color['green']>=0 && $color['green']<=($percision)) &&
				($color['red']>=0 && $color['red']<=(200+$percision)) &&
				($color['blue']>=0 && $color['blue']<=(200+$percision))){

				if(!array_key_exists($index, $colors)){
					$colors[$index]=0;
				}
				$colors[$index]++;
			}
		}
	}

	arsort($colors);
	$k_colors=array_keys($colors);
	$main_color=imagecolorsforindex($image, $k_colors[0]);

	$mask=array();
	//Выделяем самый часто используемый цвет
	$percision=30;
	for($iy=0, $y=imagesy($image); $iy<$y; $iy++){
		$mask[$iy]=array();
		for($ix=0, $x=imagesx($image); $ix<$x; $ix++){

			$index=imagecolorat($image, $ix, $iy);
			$color=imagecolorsforindex($image, $index);

			if( ($color['green']>=($main_color['green']-$percision) && $color['green']<=($main_color['green']+$percision)) &&
				($color['red']>=($main_color['red']-$percision) && $color['red']<=($main_color['red']+$percision)) &&
				($color['blue']>=($main_color['blue']-$percision) && $color['blue']<=($main_color['blue']+$percision))){

				$mask[$iy][$ix]=1;
			}else{
				$mask[$iy][$ix]=0;
			}
		}
	}

	return $mask;
}

/** Обрезать границы по горизонтали
 *
 * @param array $mask
 * @param &integer $pos_start уточняемая граница
 * @param &integer $pos_end уточняемая граница
 * @param integer $y_start известная граница по вертикали
 * @param integer $y_end известная граница по вертикали
 * @return boolean
 */
function RecogSearchSpotHorizontal($mask, &$pos_start=null, &$pos_end=null, $y_start=null, $y_end=null){
	$arr=array();

	if(is_null($y_start)){
		$y_start=0;
	}

	if(is_null($y_end)){
		$y_end=count($mask);
	}

	for($ix=0, $x=count($mask[0]); $ix<$x; $ix++){
		$cnt=0;
		for($iy=$y_start, $y=$y_end; $iy<$y; $iy++){
			if($mask[$iy][$ix]==1){
				$cnt++;
			}
		}
		$arr[$ix]=$cnt;
	}


	$arr_cnt=array();
	foreach($arr as $key=>$val){
		$cnt=$arr[$key];

		if($cnt==0){
			continue;
		}

		if(!array_key_exists($cnt, $arr_cnt)){
			$arr_cnt[$cnt]=0;
		}
		$arr_cnt[$cnt]++;
	}
	arsort($arr_cnt);
	$check_value=0;
	if(isset($arr_cnt[1]) && $arr_cnt[1]>10){
		$check_value=1;
	}

	$max_arr=array_search(max($arr), $arr);

	$pos_start=0;
	$pos_end=count($arr);
	$bl=true;
	for($i=$max_arr, $x=count($arr); $i<$x; $i++){
		if($arr[$i]==$check_value){
			if($bl==true){
				$pos_end=$i;
			}
			$bl=false;
		}else{
			$bl=true;
		}
	}

	$bl=true;
	for($i=$max_arr; $i>0; $i--){
		if($arr[$i]==$check_value){
			if($bl==true){
				$pos_start=$i+1;
			}
			$bl=false;
		}else{
			$bl=true;
		}
	}

	if(is_null($pos_start) || is_null($pos_end)) {

		return false;
	}else{
		return true;
	}

}

/** Обрезать границы по вертикали
 *
 * @param array $mask
 * @param &integer $pos_start уточняемая граница
 * @param &integer $pos_end уточняемая граница
 * @param integer $x_start известная граница по горизонтали
 * @param integer $x_end известная граница по горизонтали
 * @return boolean
 */
function RecogSearchSpotVertical($mask, &$pos_start=null, &$pos_end=null, $x_start=null, $x_end=null){
	$arr=array();
	if(is_null($x_start)){
		$x_start=0;
	}

	if(is_null($x_end)){
		$x_end=count($mask[0]);
	}

	for($iy=0, $y=count($mask); $iy<$y; $iy++){
		$cnt=0;
		for($ix=$x_start, $x=$x_end; $ix<$x; $ix++){
			if($mask[$iy][$ix]==1){
				$cnt++;
			}
		}
		$arr[$iy]=$cnt;
	}

	$pos_start=0;
	$pos_end=count($arr);

	$percision_top=2;
	$percision_bottom=2;
	for($n=10; $n>2; $n--){
		$const=$n;

		$narr=array();
		for($i=0, $x=count($arr); $i<$x; $i++){
			$summ=0;
			for($j=$i; $j<($i+$const); $j++){
				if(isset($arr[$j]))$summ+=$arr[$j];
			}

			$narr[$i]=$summ;
		}

		$max=array_search(max($narr), $narr);

		$flag=false;
		for($i=$max; $i>0; $i--){
			if(!$flag){
				if($narr[$i]<=$percision_top){
					$pos_start=$i+1;
					$flag=true;
				}
			}else{
				$arr[$i]=0;
			}
		}

		$flag=false;
		for($i=$max, $x=count($narr); $i<$x; $i++){
			if(!$flag){
				if($narr[$i]<=$percision_bottom){
					$pos_end=$i;
					$flag=true;
				}
			}else{
				$arr[$i]=0;
			}
		}
	}

	if(is_null($pos_start) || is_null($pos_end)) {
		return false;
	}

	$percision=0;
	$flag=false;
	for($i=$pos_start, $x=count($arr); $i<$x; $i++){
		if($arr[$i]>$percision){
			$pos_start=$i;
			break;

		}
	}

	$flag=false;
	for($i=$pos_end+5; $i>0; $i--){
		if(!isset($arr[$i])){
			continue;
		}
		if($arr[$i]>$percision){
			$pos_end=$i+1;
			break;
		}
	}


	if(is_null($pos_start) || is_null($pos_end)) {
		return false;
	}else{
		return true;
	}
}

/** Получить координаты фрагмента с надписью, путем уточнения границ
 *
 * @param array $image
 * @param &integer $x0 полученная координата
 * @param &integer $y0 полученная координата
 * @param &integer $x1 полученная координата
 * @param &integer $y1 полученная координата
 * @return boolean
 */
function RecogSpotGetXY($image, &$x0, &$y0, &$x1, &$y1){
	if(!RecogSearchSpotHorizontal($image, $x0, $x1)){
		return false;
	}
	if($x1<=$x0){
		return false;
	}

	//Границы по оси Y
	if(!RecogSearchSpotVertical($image, $y0, $y1, $x0, $x1)){
		return false;
	}

	if($y1<=$y0){
		return false;
	}

	//Горизонтальные границы уточняем
	if(!RecogSearchSpotHorizontal($image, $x0, $x1, $y0, $y1)){
		return false;
	}

	if($x1<=$x0){
		return false;
	}

	//Вертикальные границы уточняем
	if(!RecogSearchSpotVertical($image, $y0, $y1, $x0, $x1)){
		return false;
	}

	if($y1<=$y0){
		return false;
	}

	return true;
}
?>
